/*
 * Licensed to CRATE Technology GmbH ("Crate") under one or more contributor
 * license agreements.  See the NOTICE file distributed with this work for
 * additional information regarding copyright ownership.  Crate licenses
 * this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * However, if you have executed another commercial license agreement
 * with Crate these terms will supersede the license and you may use the
 * software solely pursuant to the terms of the relevant commercial agreement.
 */

package io.crate.action.sql.query;

import org.elasticsearch.action.ActionRequest;
import org.elasticsearch.action.ActionRequestValidationException;
import org.elasticsearch.common.io.stream.StreamInput;
import org.elasticsearch.common.io.stream.StreamOutput;
import org.elasticsearch.search.Scroll;

import java.io.IOException;

public class QueryShardScrollRequest extends ActionRequest<QueryShardScrollRequest> {

    private long id;
    private Scroll scroll;
    private int from;
    private int limit;


    public QueryShardScrollRequest(long id, Scroll scroll, int from, int limit) {
        this.id = id;
        this.scroll = scroll;
        this.from = from;
        this.limit = limit;
    }

    public QueryShardScrollRequest() {

    }

    public long id() {
        return id;
    }

    public Scroll scroll() {
        return scroll;
    }

    public int limit() {
        return limit;
    }

    public int from() {
        return from;
    }

    @Override
    public ActionRequestValidationException validate() {
        return null;
    }

    @Override
    public void readFrom(StreamInput in) throws IOException {
        super.readFrom(in);
        id = in.readLong();
        scroll = Scroll.readScroll(in);
        from = in.readInt();
        limit = in.readInt();
    }

    @Override
    public void writeTo(StreamOutput out) throws IOException {
        super.writeTo(out);
        out.writeLong(id);
        scroll.writeTo(out);
        out.writeInt(from);
        out.writeInt(limit);
    }
}
